#!/usr/bin/env python
# -*- coding: utf-8  -*-
# Script en Python que organiza los ficheros del directorio
# de trabajo en función de su nombre completo.
 
# USO CORRECTO: python pygments_script.py [-v] 
#               [-h|-f|-e <extension>|-c|-d|-o <dir_salida>]
#               <fich_o_dir_entrada> ...
#   * -v -> modo chivato activado
#   * -h o --help -> ayuda
#   * -f o --force -> fuerza la reescritura de los ficheros,
#                     si existen se borran
#   * -e o --ext -> lee la extensión de los ficheros que tomará
#                   como entrada
#   * -c o --clean -> limpia los ficheros que se den como entrada
#   * -d o --dirclean -> limpia los directorios que se den
#   * -o o --output -> genera los nuevos ficheros en un directorio
#                      específico, en lugar de en su misma ruta
#   * -l o --longname -> los nuevos ficheros tendrán un nombre
#                        del tipo: nombre_ext.tex siendo "ext"
#                        la extensión antigua
#                        Esto resulta útil para proyectos con
#                        ficheros del tipo a.h y a.c o a.cpp

# Creado por Pablo Recio Quijano y Noelia Sales Montes

# This program is free software: you can redistribute
# it and/or modify it under the terms of the GNU General
# Public License as published by the Free Software
# Foundation, either version 3 of the License, or any
# later version.

# This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.

# You should have received a copy of the GNU General 
# Public License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.
  
# Contacto:  
#            pablo.recioquijano@alum.uca.es
#            noelia.salesmontes@gmail.com 

import sys
import getopt
import os
import glob

from pygments import highlight # Función para hacer el highlighting
from pygments.lexers import guess_lexer_for_filename # Adivinar qué
# lexer debe usar, por si pasamos distintos ficheros
from pygments.formatters import LatexFormatter # Formato de salida = LaTeX
from pygments.lexers import MoinWikiLexer # Lexer para formato wiki

verbose=False # Modo chivato
force=False # Fuerza la reescritura de los documentos
active_dir=False # Directorios completos en lugar de fichero a fichero
clear=False # Limpieza de un listado de ficheros
clear_dir=False # Limpieza de los TEX de un(os) directorio(s)
ext="" # Extensión
output_dir="" # Directorio donde se generarán los nuevos ficheros
              # Si no se introduce ninguno, se generan en la misma
              # ruta del fichero
longname=False # Generación de nombres de salida largos

# Impresión del uso del script
def usage():
    print "USO CORRECTO: python pygments_script.py [-v] [-h|-f|-e <extension>|-c|-d|-o <dir_salida>|-l] <fich_o_dir_entrada> ..."

# Borrado de un único fichero
def remove(file):
    try:
        os.remove(file)
    except:
        verb("No se ha podido borrar el fichero %s" % file)

# Modo chivato
def verb(cad):
    if verbose == True:
        print cad

# Obtención del nombre del nuevo fichero
def name(input_file):
    head, file = os.path.split(input_file)
    name = file[:file.find('.')] # Nombre sin extensión
    ext = file[file.find('.') + 1:]

    # Tipo de nombre a generar
    if longname == True:
        name = name + '_' + ext
    verb("NAME: %s" % name)

    # Control del directorio donde generar los nuevos ficheros
    if output_dir == "":
        return os.path.join(head, name + '.tex') 
    else:
        return os.path.join(output_dir, name + '.tex')


# Utiliza pygments sobre un único fichero
def pygmentize_single_file(input_file, output_file):
    f = open(input_file, "r")
    code = f.read() # Cargamos todo el código
    f.close()

    try:
        lexer = guess_lexer_for_filename(input_file, code) # Hallamos el lexer
    except:
        if ext == 'wik':
            lexer = MoinWikiLexer()
        else:
            verb("No se ha encontrado el lexer para %s" % input_file)

    if force == True: # Forzar la reescritura del fichero
        remove(output_file)
        verb("Forzando la reescritura del fichero %s" % output_file)
    
    result = highlight(code, lexer,
                       LatexFormatter(linenos=True, encoding='utf-8'))
    f = open(output_file, "w")
    f.write(result)

# Utilización de pygments sobre una lista de ficheros
def pygmentize_file_list(input_files):
    for input_file_complete in input_files:
        verb("Pygments sobre fichero: %s" % input_file_complete)
        output_file = name(input_file_complete)
        pygmentize_single_file(input_file_complete, output_file)

# Utilización de pygments sobre los ficheros de una determinada
# extensión que se encuentren en un directorio
def pygmentize_dir(input_dirs, extension):
    for dir in input_dirs:
        if not os.path.exists(dir):
            if verbose == True: # Modo chivato
                print "No existe el directorio %s" % dir
        else: # Si existe el directorio
            e = os.path.join(dir,"*." + extension) # Expresión regular
            if verbose == True: # Modo chivato
                print "Expresión regular: %s" % e
            # Encuentra los ficheros que terminen en la extension
            for input_file in glob.glob(e):
                output_file = name(input_file)
                verb("Pygments sobre fichero: %s" % input_file)
                verb("INPUT: %s" % input_file)
                verb("OUTPUT: %s" % output_file)
                pygmentize_single_file(input_file, output_file)
        
# Limpia los .tex dentro de los directorios introducidos
def clean_dir(dirs):
    for dir in dirs:
        if not os.path.exists(dir):
            verb("No existe el directorio %s" % dir)
        else:
            e = os.path.join(dir,"*.tex") # Expresión regular
            verb("Expresión regular (TEX): %s" % e)
            # Encuentra los ficheros que terminen en la extension
            for f in glob.glob(e):
                verb("Fichero a borrar: %s" % f)
                remove(f)

# Limpia la lista de ficheros introducidos
def clean(files):
    for f in files:
        if not os.file.exists(f):
            verb("No existe el fichero %s" % f)
        else:
            remove(f)    

# Función principal
if __name__ == "__main__":
    try:
        opts, args = getopt.getopt(sys.argv[1:],
                                   'hvfe:cdo:l',["help","verbose","force",
                                                 "ext","clean","dirclean",
                                                 "output","longname"])
    except getopt.GetoptError, err:
        # Imprimir error y ayuda y salir
        print str(err)
        usage()
        sys.exit(2)
        

    # Identificar opciones
    for o, a in opts:
        if o in ("-v", "--verbose"):
            verbose = True
        elif o in ("-h", "--help"):
            usage()
            sys.exit()
        elif o in ("-f", "--force"):
            force = True
        elif o in ("-e", "--ext"):
            active_dir = True
            ext = a
        elif o in ("-c", "--clean"):
            clear = True
        elif o in ("-d", "--dirclean"):
            clear_dir = True
        elif o in ("-o", "--output"):
            output_dir = a
        elif o in ("-l", "--longname"):
            longname = True
        else:
            assert False, "Opción no manejada"
            
    # Llamadas a las funciones
    if len(args) == 0:
        print "Este script necesita argumentos"
        usage()
        sys.exit(2)
    else:
        if clear == True:
            clean(args)
        elif clear_dir == True:
            clean_dir(args)
        elif active_dir == True:
            if ext == "":
                ext = "wik"
            pygmentize_dir(args,ext)
        else:
            pygmentize_file_list(args)
